@@ -1,12 +1,14 @@
-use regex::Regex;
+//! Describes a `RINEX` header, includes
+//! rinex header parser and associated methods
+//use regex::Regex;
 use thiserror::Error;
-use chrono::Timelike;
 
+use crate::version;
 use crate::gnss_time;
-use crate::version::RinexVersion;
-use crate::observation::ObservationType;
 use crate::{is_rinex_comment, RinexType, RinexTypeError};
 use crate::constellation::{Constellation, ConstellationError};
+use crate::meteo::{MeteoDataCode, MeteoDataCodeError};
+use crate::observation::{ObservationCode, ObservationCodeError};
 
 /// Describes a `CRINEX` (compact rinex) 
 pub const CRINEX_MARKER_COMMENT : &str = "COMPACT RINEX FORMAT";
@@ -15,7 +17,7 @@ pub const HEADER_END_MARKER : &str = "END OF HEADER";
 
 /// GNSS receiver description
 #[derive(Debug)]
-struct Rcvr {
+pub struct Rcvr {
     model: String, 
     sn: String, // serial #
     firmware: String, // firmware #
@@ -37,7 +39,7 @@ impl std::str::FromStr for Rcvr {
     fn from_str (line: &str) -> Result<Self, Self::Err> {
         let (id, rem) = line.split_at(20);
         let (make, rem) = rem.split_at(20);
-        let (version, rem) = rem.split_at(20);
+        let (version, _) = rem.split_at(20);
         Ok(Rcvr{
             sn: String::from(id.trim()),
             model: String::from(make.trim()),
@@ -48,13 +50,19 @@ impl std::str::FromStr for Rcvr {
 
 /// Antenna description 
 #[derive(Debug)]
-struct Antenna {
+pub struct Antenna {
+    /// Hardware model / make descriptor
     model: String,
-    sn: String, // serial #
-    coords: Option<rust_3d::Point3D>, // ref. point position
-    height: Option<f32>, // height in comparison to ref. point
-    eastern_eccentricity: Option<f32>, // in comparison to ref. point
-    northern_eccentricity: Option<f32>, // in comparison to ref. point
+    /// Serial number / identification number
+    sn: String,
+    /// 3D coordinates of reference point
+    coords: Option<rust_3d::Point3D>,
+    /// height in comparison to ref. point
+    height: Option<f32>,
+    /// eastern eccentricity compared to ref. point
+    eastern_eccentricity: Option<f32>,
+    /// northern eccentricity compare to ref. point
+    northern_eccentricity: Option<f32>,
 }
 
 impl Default for Antenna {
@@ -75,7 +83,7 @@ impl std::str::FromStr for Antenna {
     type Err = std::io::Error;
     fn from_str (line: &str) -> Result<Self, Self::Err> {
         let (id, rem) = line.split_at(20);
-        let (make, rem) = rem.split_at(20);
+        let (make, _) = rem.split_at(20);
         Ok(Antenna{
             sn: String::from(id.trim()),
             model: String::from(make.trim()),
@@ -99,20 +107,18 @@ impl Antenna {
 }
 
 /// `LeapSecond` to describe leap seconds.
-/// leap: current number of leap seconds 
-/// past_future: future or past leap seconds ΔtLS,   
-/// ie., future leap second if week and day number are in future   
-/// week: week counter sometimes present.   
-/// day: week counter sometimes present.   
-/// GnssTimes:   
 /// GLO = UTC = GPS - ΔtLS   
 /// GPS = GPS = UTC + ΔtLS   
 #[derive(Copy, Clone, Debug)]
 pub struct LeapSecond {
-    leap: u32, // current number
-    past_future: u32, // future or past leap seconds ΔtLS   
-    week: u32, // week number 
-    day: u32, // day number
+    /// current number
+    leap: u32,
+    /// future or past leap seconds (ΔtLS)   
+    past_future: u32,
+    /// week number
+    week: u32,
+    /// day number
+    day: u32,
 }
 
 impl Default for LeapSecond {
@@ -143,13 +149,12 @@ impl std::str::FromStr for LeapSecond {
                 let (leap, rem) = s.split_at(6);
                 let (past, rem) = rem.split_at(6);
                 let (week, rem) = rem.split_at(6);
-                let (day, rem) = rem.split_at(6);
+                let (day, _) = rem.split_at(6);
                 ls.leap = u32::from_str_radix(leap.trim(),10)?;
                 ls.past_future = u32::from_str_radix(past.trim(),10)?;
                 ls.week = u32::from_str_radix(week.trim(),10)?;
                 ls.day = u32::from_str_radix(day.trim(),10)?
             },
-            _ => return Err(Error::LeapSecondParsingError(String::from(s)))
         }
         Ok(ls)
     }
@@ -157,7 +162,7 @@ impl std::str::FromStr for LeapSecond {
 
 /// Describes `Compact RINEX` specific information
 #[derive(Debug)]
-struct CrinexInfo {
+pub struct CrinexInfo {
     version: String, // compression version
     prog: String, // compression program
     date: chrono::NaiveDateTime, // date of compression
@@ -165,79 +170,96 @@ struct CrinexInfo {
 
 /// Describes known marker types
 enum MarkerType {
-    Geodetic, // earth fixed high precision
-    NonGeodetic, // earth fixed low prcesion
-    NonPhysical, // generated from network
-    Spaceborne, // orbiting space vehicule
-    Airborne, // aircraft, balloon, ...
-    WaterCraft, // mobile water craft
-    GroundCraft, // mobile terrestrial vehicule
-    FixedBuoy, // fixed on water surface
-    FloatingBuoy, // floating on water surface
-    FloatingIce, // floating on ice sheet
-    Glacier, // fixed on a glacier
-    Ballistic, // rockets, shells, etc..
-    Animal, // animal carrying a receiver
-    Human, // human being
-}
-
-/// Clock Data types
-enum ClockDataType {
-    ClockDataAr, // recvr clocks derived from a network
-    ClockDataAs, // sat clocks derived from a network
-    ClockDataCr, // Calibration measurements for a single GNSS rcvr
-    ClockDataDr, // Discontinuity measurements for a single GPS receiver
-    ClockDataMs, // Monitor measurements for broadcast sat clocks
-}
-
-enum SignalStrength {
-    DbHz12, // < 12 dBc/Hz
-    DbHz12_17, // 12 <= x < 17 dBc/Hz
-    DbHz18_23, // 18 <= x < 23 dBc/Hz
-    DbHz21_29, // 24 <= x < 29 dBc/Hz
-    DbHz30_35, // 30 <= x < 35 dBc/Hz
-    DbHz36_41, // 36 <= x < 41 dBc/Hz
-    DbHz42_47, // 42 <= x < 47 dBc/Hz
-    DbHz48_53, // 48 <= x < 53 dBc/Hz
-    DbHz54, // >= 54 dBc/Hz 
+    /// Earth fixed & high precision
+    Geodetic,
+    /// Earth fixed & low precision
+    NonGeodetic,
+    /// Generated from network
+    NonPhysical,
+    /// Orbiting space vehicule
+    Spaceborne,
+    /// Aircraft, balloon..
+    Airborne,
+    /// Mobile water craft
+    WaterCraft,
+    /// Mobile terrestrial vehicule
+    GroundCraft,
+    /// Fixed on water surface
+    FixedBuoy,
+    /// Floating on water surface
+    FloatingBuoy,
+    /// Floating on ice
+    FloatingIce, 
+    /// Fixed on glacier
+    Glacier,
+    /// Rockets, shells, etc..
+    Ballistic,
+    /// Animal carrying a receiver
+    Animal,
+    /// Human being carrying a receiver
+    Human,
 }
 
 /// Describes `RINEX` file header
 #[derive(Debug)]
 pub struct RinexHeader {
-    version: RinexVersion, // version description
-    crinex: Option<CrinexInfo>, // if this is a CRINEX
-    rinex_type: RinexType, // type of Rinex
-    constellation: Constellation, // GNSS constellation being used
-    program: String, // program name 
-    run_by: String, // program run by
-    //date: strtime, // file date of creation
-    station: String, // station label
-    station_id: String, // station id
-    station_url: Option<String>, // station url
-    observer: String, // observer
-    agency: String, // agency
-    rcvr: Option<Rcvr>, // optionnal hardware infos
-    ant: Option<Antenna>, // optionnal antenna infos
-    leap: Option<LeapSecond>, // optionnal leap seconds infos
-    coords: Option<rust_3d::Point3D>, // station approx. coords
-    wavelengths: Option<(u32,u32)>, // observations wavelengths
-    sampling_interval: Option<f32>, // sampling
-    epochs: (Option<gnss_time::GnssTime>, Option<gnss_time::GnssTime>), // first , last observations
-    license: Option<String>, // optionnal license
-    doi: Option<String>, // optionnal Object Idenfier ("IoT")
-    gps_utc_delta: Option<u32>, // optionnal GPS / UTC time difference
-    // processing
-    scaling_factor: Option<f64>, // optionnal data scaling
+    /// revision for this `RINEX`
+    pub version: version::Version, 
+    /// optionnal `CRINEX` (compact `RINEX` infos), 
+    /// if this is a CRINEX
+    pub crinex: Option<CrinexInfo>, 
+    /// type of `RINEX` file
+    pub rinex_type: RinexType, 
+    /// `GNSS` constellation being used
+    pub constellation: Constellation, 
+    /// program name
+    pub program: String, 
+    /// program `run by`
+    pub run_by: String, // program run by
+    /// station label
+    pub station: String, 
+    /// station identifier
+    pub station_id: String, 
+    /// optionnal station URL 
+    pub station_url: Option<String>, 
+    /// name of observer
+    pub observer: String, 
+    /// name of production agency
+    pub agency: String, 
+    /// optionnal hardware (receiver) infos
+    pub rcvr: Option<Rcvr>, 
+    /// optionnal antenna infos
+    pub ant: Option<Antenna>, 
+    /// optionnal leap seconds infos
+    pub leap: Option<LeapSecond>, 
+    /// station approxiamte coordinates
+    pub coords: Option<rust_3d::Point3D>, 
+    /// optionnal observation wavelengths
+    pub wavelengths: Option<(u32,u32)>, 
+    /// optionnal sampling interval (s)
+    pub sampling_interval: Option<f32>, 
+    /// optionnal (first, last) epochs
+    pub epochs: (Option<gnss_time::GnssTime>, Option<gnss_time::GnssTime>),
+    /// optionnal file license
+    pub license: Option<String>,
+    /// optionnal Object Identifier (IoT)
+    pub doi: Option<String>,
+    /// optionnal GPS/UTC time difference
+    pub gps_utc_delta: Option<u32>,
+    /// processing:   
+    /// optionnal data scaling
+    pub data_scaling: Option<f64>,
     // optionnal ionospheric compensation param(s)
     //ionospheric_corr: Option<Vec<IonoCorr>>,
     // possible time system correction(s)
     //gnsstime_corr: Option<Vec<gnss_time::GnssTimeCorr>>,
-    // true if epochs & data compensate for local clock drift 
-    rcvr_clock_offset_applied: Option<bool>, 
+    /// Observations:   
+    /// true if epochs & data compensate for local clock drift 
+    pub rcvr_clock_offset_applied: Option<bool>, 
     // observation (specific)
-    // obs_types: lists all types of observations contained in this `Rinex`
-    obs_types: Option<std::collections::HashMap<Constellation, Vec<ObservationType>>>,
+    /// lists all types of observations 
+    /// contained in this `Rinex` OBS file
+    pub obs_codes: Option<Vec<String>>, 
 }
 
 #[derive(Error, Debug)]
@@ -262,12 +284,16 @@ pub enum Error {
     DateParsingError(#[from] chrono::ParseError),
     #[error("failed to parse leap second from \"{0}\"")]
     LeapSecondParsingError(String),
+    #[error("observation type code not recognized \"{0}\"")]
+    ObservationTypeCodeError(#[from] ObservationCodeError),
+    #[error("meteo data code not recognized \"{0}\"")]
+    MeteoDataCodeError(#[from] MeteoDataCodeError),
 }
 
 impl Default for RinexHeader {
     fn default() -> RinexHeader {
         RinexHeader {
-            version: RinexVersion::default(), 
+            version: version::Version::default(), 
             crinex: None,
             rinex_type: RinexType::default(),
             constellation: Constellation::default(),
@@ -290,10 +316,10 @@ impl Default for RinexHeader {
             // observations
             epochs: (None, None),
             wavelengths: None,
-            obs_types: None,
+            obs_codes: None,
             // processing
             rcvr_clock_offset_applied: None,
-            scaling_factor: None,
+            data_scaling: None,
             //ionospheric_corr: None,
             //gnsstime_corr: None,
             sampling_interval: None,
@@ -346,10 +372,10 @@ impl std::str::FromStr for RinexHeader {
         // line1 {} {} {} // label [VERSION/TYPE/GNSS] 
         let (version_str, remainder) = line.split_at(20);
         let (type_str, remainder) = remainder.trim().split_at(20);
-        let (constellation_str, remainder) = remainder.trim().split_at(20);
+        let (constellation_str, _) = remainder.trim().split_at(20);
 
         let rinex_type = RinexType::from_str(type_str.trim())?;
-        let mut constellation: Constellation;
+        let constellation: Constellation;
         
         if type_str.trim().contains("GLONASS") {
             // special case, sometimes GLONASS NAV
@@ -359,7 +385,7 @@ impl std::str::FromStr for RinexHeader {
             constellation = Constellation::from_str(constellation_str.trim())?
         }
 
-        let version = RinexVersion::from_str(version_str.trim())?;
+        let version = version::Version::from_str(version_str.trim())?;
         if !version.is_supported() {
             return Err(Error::VersionNotSupported(String::from(version_str)))
         }
@@ -406,6 +432,7 @@ impl std::str::FromStr for RinexHeader {
             }
         };
         // identify date format (YMDHMS)
+/*
         println!("date str \"{}\"", date_str);
         let regex: Vec<Regex> = vec![
             Regex::new(r"\d\d\d\d\d\d\d\d \d\d:\d\d:\d\d$")
@@ -418,7 +445,6 @@ impl std::str::FromStr for RinexHeader {
             "%Y%m%d %H%M%S",
         ];
 
-/*
          * for i in 0..regex.len() {
             if regex[i].is_match(cleanedup) {
                 if is_utc {
@@ -457,11 +483,9 @@ impl std::str::FromStr for RinexHeader {
         let mut rcvr_clock_offset_applied: Option<bool> = None;
         let mut coords     : Option<rust_3d::Point3D> = None;
         let mut epochs: (Option<gnss_time::GnssTime>, Option<gnss_time::GnssTime>) = (None, None);
-        // RinexType::ObservationData
-        let mut obs_nb_sat : u32 = 0;
-        let mut obs_types: 
-            Option<std::collections::HashMap<Constellation, Vec<ObservationType>>> 
-                = None;
+        // (OBS) 
+        let obs_nb_sat : u32 = 0;
+        let mut obs_codes  : Vec<String> = Vec::new();
 
         loop {
             /*
@@ -478,7 +502,7 @@ impl std::str::FromStr for RinexHeader {
                 station_id = String::from(line.split_at(20).0.trim()) 
             } else if line.contains("OBSERVER / AGENCY") {
                 let (obs, ag) = line.split_at(20);
-                let ag = ag.split_at(20).0;
+                observer = String::from(obs.trim());
                 agency = String::from(ag.trim())
 
             } else if line.contains("REC # / TYPE / VERS") {
@@ -579,17 +603,17 @@ impl std::str::FromStr for RinexHeader {
             } else if line.contains("# OF SATELLITES") {
                 // will always appear prior PRN/#OBS
                 // determines nb of satellites in observation file
-                let (nb, _) = line.split_at(10);
-                obs_nb_sat = u32::from_str_radix(nb.trim(), 10)?
+                //let (nb, _) = line.split_at(10);
+                //obs_nb_sat = u32::from_str_radix(nb.trim(), 10)?
 
             } else if line.contains("PRN / # OF OBS") {
-                let (sv, rem) = line.split_at(7);
+                let (sv, _) = line.split_at(7);
                 if sv.trim().len() > 0 {
                     
                 }
                 // lists all Sv
-                let items: Vec<&str> = line.split_ascii_whitespace()
-                    .collect();
+                //let items: Vec<&str> = line.split_ascii_whitespace()
+                //    .collect();
                  
             } else if line.contains("SYS / PHASE SHIFT") {
                 //TODO
@@ -604,24 +628,53 @@ impl std::str::FromStr for RinexHeader {
                 // RINEX::ClockData specific 
                 // + number of different clock data types stored
                 // + list of clock data  types
-            } else if line.contains("TYPES OF OBS") {
-                // Rinex::ObservationData specific
-                // types of observation enumeration
-                // but it's not a per constellation enumeration
-                // => we set a flag and we will post process
-                //    to provide high level information in the end (sorting capacity)
-                //5    L1    L2    C1    P1    P2                        # / TYPES OF OBSERV
-                /*unspecified_obs_system = true;
-                let (num, rem) = line.split_at(6);
-                for i in 0..num {
-                    let (code, rem) = rem.split_at(6);
-                    if code.trim().len() == 0 {
-                        break
-                    }
-                    
-                }*/
-
+            } else if line.contains("TYPES OF OBS") { 
+                // Rinex V < 3 : old fashion obs data
+                // ⚠ ⚠ could either be observation or meteo data
+                let (rem, _) = line.split_at(60);
+                match rinex_type {
+                    RinexType::ObservationData => {
+                        let (_, mut rem) = rem.split_at(6);
+                        loop {
+                            let (code, r) = rem.split_at(6);
+                            if code.trim().len() == 0 {
+                                break
+                            }
+                            obs_codes.push(code.trim().to_string());
+                            if r.len() == 0 {
+                                break
+                            }
+                            rem = r
+                        }
+                    },
+                    RinexType::MeteorologicalData => {
+                        let (_, mut rem) = rem.split_at(6);
+                        loop {
+                            let (code, r) = rem.split_at(6);
+                            if code.trim().len() == 0 {
+                                break
+                            }
+                            obs_codes.push(code.trim().to_string());
+                            if r.len() == 0 {
+                                break
+                            }
+                            rem = r
+                        }
+                    },
+                    _ => {},
+                }
             } else if line.contains("SYS / # / OBS TYPES") {
+                //let content = line.split_at(60).0;
+/*G   22 C1C L1C D1C S1C C1W S1W C2W L2W D2W S2W C2L L2L D2L  SYS / # / OBS TYPES
+       S2L C5Q L5Q D5Q S5Q C1L L1L D1L S1L                  SYS / # / OBS TYPES
+E   20 C1C L1C D1C S1C C6C L6C D6C S6C C5Q L5Q D5Q S5Q C7Q  SYS / # / OBS TYPES
+       L7Q D7Q S7Q C8Q L8Q D8Q S8Q                          SYS / # / OBS TYPES
+S    8 C1C L1C D1C S1C C5I L5I D5I S5I                      SYS / # / OBS TYPES
+R   20 C1C L1C D1C S1C C1P L1P D1P S1P C2P L2P D2P S2P C2C  SYS / # / OBS TYPES
+       L2C D2C S2C C3Q L3Q D3Q S3Q                          SYS / # / OBS TYPES
+C   20 C1P L1P D1P S1P C5P L5P D5P S5P C2I L2I D2I S2I C7I  SYS / # / OBS TYPES
+       L7I D7I S7I C6I L6I D6I S6I                          SYS / # / OBS TYPES
+I    4 C5A L5A D5A S5A                                      SYS / # / OBS TYPES*/
                 // Rinex::ObservationData 
                 // types of observation for specified constellation
                 
@@ -635,15 +688,6 @@ impl std::str::FromStr for RinexHeader {
                 } else {
                     println!("REM \"{}\"", rem.trim());
                 }
-//G   22 C1C L1C D1C S1C C1W S1W C2W L2W D2W S2W C2L L2L D2L  SYS / # / OBS TYPES
-//       S2L C5Q L5Q D5Q S5Q C1L L1L D1L S1L                  SYS / # / OBS TYPES
-//E   20 C1C L1C D1C S1C C6C L6C D6C S6C C5Q L5Q D5Q S5Q C7Q  SYS / # / OBS TYPES
-//       L7Q D7Q S7Q C8Q L8Q D8Q S8Q                          SYS / # / OBS TYPES
-//S    8 C1C L1C D1C S1C C5I L5I D5I S5I                      SYS / # / OBS TYPES
-            } else if line.contains("TYPES OF OBS") || 
-                line.contains("SYS / # / OBS TYPES") {
-     //5    L1    L2    C1    P1    P2                        # / TYPES OF OBSERV
-            
             } else if line.contains("SIGNAL STRENGHT UNIT") {
                 //TODO
             } else if line.contains("INTERVAL") {
@@ -698,6 +742,11 @@ impl std::str::FromStr for RinexHeader {
                 ant.as_mut().unwrap().set_northern_eccentricity(ant_hen.unwrap().2);
             }
         }
+
+        let obs_codes: Option<Vec<String>> = match obs_codes.len() {
+            0 => None,
+            _ => Some(obs_codes)
+        };
         
         Ok(RinexHeader{
             version: version,
@@ -722,10 +771,10 @@ impl std::str::FromStr for RinexHeader {
             gps_utc_delta: None,
             // observations
             epochs: epochs,
-            obs_types: None,
+            obs_codes,
             // processing
             sampling_interval: sampling_interval,
-            scaling_factor: None,
+            data_scaling: None,
             //ionospheric_corr: None,
             //gnsstime_corr: None,
             rcvr_clock_offset_applied,
@@ -758,14 +807,4 @@ impl RinexHeader {
             _ => None,
         }
     }
-
-    /// Returns `Rinex` Version number
-    pub fn get_rinex_version (&self) -> RinexVersion { self.version }
-    /// Returns `Rinex` type 
-    pub fn get_rinex_type (&self) -> RinexType { self.rinex_type }
-    /// Returns `GNSS` constellation
-    pub fn get_constellation (&self) -> Constellation { self.constellation }
-
-    /// Returns `LeapSecond` infos (if any)
-    pub fn get_leap_second (&self) -> Option<LeapSecond> { self.leap }
 }
